@@ -10,10 +10,10 @@ from unicodedata import decimal
 import warnings
 import copy
 from datetime import datetime
-import time
-import random
-import json
-import re
+import time, random, json, re
+import hmac
+import urllib
+import base64, hashlib
 
 from typing import Union, Pattern
 
@@ -127,7 +127,7 @@ def send_email(from_addr: str, to_addrs: str, subject: str, text_body: str = ''
             'port': 465,
             'user': 'henrytian@163.com',
             'pwd': '123456',
-            'type': 'plain'         # plain (default) / ssl / tls
+            'type': 'ssl'         # plain / ssl (default) / tls
         }
         debug {bool} -- If True output debug info.
         send_to_file {str} -- File path for writing email info to text file.
@@ -145,14 +145,14 @@ def send_email(from_addr: str, to_addrs: str, subject: str, text_body: str = ''
 
     if send_to_file:
         if not email_file_dir:
-            email_file_dir = os.path.join(os.getcwd(), 'logs')
+            email_file_dir = os.path.join(os.getcwd(), '.logs')
         if not os.path.exists(email_file_dir):
             os.mkdir(email_file_dir)
 
     #TODO: Use schema to validate smtp_config
 
     from email.message import EmailMessage
-    from email.utils import make_msgid
+    from email.utils import make_msgid, localtime
     from mimetypes import guess_type
 
     msg = EmailMessage()
@@ -160,6 +160,7 @@ def send_email(from_addr: str, to_addrs: str, subject: str, text_body: str = ''
     msg['From'] = from_addr
     msg['To'] = to_addrs
     msg['Subject'] = subject.replace('\n', '').replace('\r', '')
+    msg['Date'] = localtime()
 
     # set the plain text body
     msg.set_content(text_body)
@@ -673,4 +674,54 @@ def env():
 
 
 def is_prod():
-    return env() == 'prod'
\ No newline at end of file
+    return env() == 'prod'
+
+
+def create_sign_for_dingtalk(secret: str):
+    """
+    docstring
+    """
+    timestamp = str(round(time.time() * 1000))
+    secret_enc = secret.encode('utf-8')
+    string_to_sign = '{}\n{}'.format(timestamp, secret)
+    string_to_sign_enc = string_to_sign.encode('utf-8')
+    hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
+    sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
+    return timestamp, sign
+
+
+def help():
+    import inspect
+    local_vars = locals()
+    for name, obj in local_vars.items():
+        if inspect.isfunction(obj):
+            signature = inspect.signature(obj)
+            params = [str(param) for param in signature.parameters.values()]
+            print(f"Function: {name}, Parameters: {', '.join(params)}")
+            
+def get_cached_file(file_path, cache_time=3600, clear_cache=False):
+    """
+    检查缓存文件是否有效
+    
+    :param file_name: 目标文件名
+    :param cache_dir: 缓存目录(默认当前目录下的.cache)
+    :param cache_time: 有效期秒数(默认3600秒)
+    :return: 有效返回完整文件路径，否则返回None
+    """
+   
+    # 检查文件是否存在
+    if not os.path.exists(file_path):
+        return None
+    
+    # 获取文件修改时间和当前时间
+    last_modified = os.path.getmtime(file_path)
+    current_time = time.time()
+    
+    # 检查是否在有效期内
+    if (current_time - last_modified) > cache_time:
+        if clear_cache:
+            os.remove(file_path)
+        return None
+    
+    # 返回绝对路径
+    return os.path.abspath(file_path)
\ No newline at end of file
